using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace Miroslaw.APCF.PF
{
    public partial class MainForm : Form
    {
        NullEditor _nullEditor = new NullEditor();
        Iface.Loader _loader = new Iface.Loader();
        AssElt _curAss;

        AssBndList _assembled = new AssBndList();

        public MainForm()
        {
            InitializeComponent();

            _loader.LoadFactories(Assembly.GetExecutingAssembly());
            _lstProduct.BeginUpdate();
            foreach (Iface.IProductEditor editor in _loader.GetEditors())
            {
                int item = _lstProduct.Items.Add(new ProdElt (editor));
            }
            _lstProduct.EndUpdate();

            _nullEditor.Dock = DockStyle.Fill;
            _nullEditor.Visible = true;
            _panelEditor.Controls.Add(_nullEditor);

            _lstAssembly.DataSource = _assembled;
            _lstAssembly.DisplayMember = "Name";
        }

        void _selectEditor(Iface.IProductEditor editor)
        {
            Control c = _panelEditor.Controls[0];
            if (_curAss != null)
            {
                _assembled.FireChanged();
                _curAss = null;
            }

            _panelEditor.Controls.RemoveAt(0);
            c.Visible = false;
            Iface.IProductEditor e = (Iface.IProductEditor)c;
            e.Edit(null);

            c = editor == null ? _nullEditor : (Control)editor;
            c.Dock = DockStyle.Fill;
            c.Visible = true;
            _panelEditor.Controls.Add(c);
        }

        void _selectProduct(Iface.Product product)
        {
            Iface.IProductEditor editor = _loader.GetEditor(product.Guid);
            _selectEditor(editor);
            editor.Edit(product);
        }

        bool _suppresSelection;
        private void _lstProduct_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_suppresSelection)
                return;

            _suppresSelection = true;


            ProdElt pe = (ProdElt)_lstProduct.SelectedItem;
            Iface.IProductEditor editor = pe != null ? pe.Pe : null;
            _selectEditor(editor);
            if (editor != null)
            editor.Edit(null);

            _lstAssembly.SelectedIndex = -1;

            _suppresSelection = false;
        }

        private void _lstAssembly_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (_suppresSelection)
                return;

            _suppresSelection = true;

            AssElt ae = (AssElt)_lstAssembly.SelectedItem;
            if (ae == null)
                _selectEditor(null);
            else
            {
                _selectProduct(ae.P);
                _curAss = ae;
                ae.FireNameChanged();
            }

            _lstProduct.SelectedIndex = -1;

            _suppresSelection = false;
        }

        private void _createProduct(Iface.IProductEditor pe)
        {
            Iface.Product product = pe.Create();
            _assembled.Add(new AssElt(product));
        }

        private void _lstProduct_DoubleClick(object sender, EventArgs e)
        {
            ProdElt pe = (ProdElt)_lstProduct.SelectedItem;
            if (pe != null)
                _createProduct(pe.Pe);
        }

        private void _lstProduct_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                ProdElt pe = (ProdElt)_lstProduct.SelectedItem;
                if (pe != null)
                    _createProduct(pe.Pe);
            }
        }

        private void _lstAssembly_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Delete)
            {
                int idx = _lstAssembly.SelectedIndex;
                if (idx >= 0)
                    _lstAssembly.Items.RemoveAt(idx);
            }
        }

        #region ListElements
        class ProdElt
        {
            private readonly Iface.IProductEditor _pe;

            public Iface.IProductEditor Pe
            {
                get { return _pe; }
            } 

            public ProdElt(Iface.IProductEditor pe)
            {
                _pe = pe;
            }

            public override string ToString()
            {
                return _pe.Name;
            }
        }
        class AssElt
        {
            private readonly Iface.Product _p;

            public Iface.Product P
            {
                get { return _p; }
            } 

            public AssElt(Iface.Product p)
            {
                _p = p;
            }

            public event EventHandler NameChanged;
            public string Name
            {
                get { return _p.Name; }
            }
            public void FireNameChanged()
            {
                if (NameChanged != null)
                    NameChanged(this, EventArgs.Empty);
            }


            public override string ToString()
            {
                return _p.Name;
            }
        }
        #endregion

        #region AssBndList
        private class AssBndList : System.Collections.CollectionBase, IBindingList
        {
            public void Add(AssElt elt)
            {
                List.Add(elt);
            }

            protected override void OnInsertComplete(int index, object value)
            {
                base.OnInsertComplete(index, value);
                if (ListChanged != null)
                    ListChanged (this, new ListChangedEventArgs (ListChangedType.ItemAdded, index));
            }

            protected override void OnClearComplete()
            {
                base.OnClearComplete();
                if (ListChanged != null)
                    ListChanged (this, new ListChangedEventArgs (ListChangedType.Reset, -1));
            }

            protected override void OnRemoveComplete(int index, object value)
            {
                base.OnRemoveComplete(index, value);
                if (ListChanged != null)
                    ListChanged (this, new ListChangedEventArgs (ListChangedType.ItemDeleted, index));
            }

            protected override void OnSetComplete(int index, object oldValue, object newValue)
            {
                base.OnSetComplete(index, oldValue, newValue);
                if (ListChanged != null)
                    ListChanged(this, new ListChangedEventArgs(ListChangedType.ItemChanged, index));
            }

            public void FireChanged()
            {
                if (ListChanged != null)
                    ListChanged(this, new ListChangedEventArgs(ListChangedType.Reset, -1));
            }

            #region IBindingList Member

            public void AddIndex(PropertyDescriptor property)
            {
                throw new Exception("The method or operation is not implemented.");
            }

            public object AddNew()
            {
                return false;
            }

            public bool AllowEdit
            {
                get { return false; }
            }

            public bool AllowNew
            {
                get { return false; }
            }

            public bool AllowRemove
            {
                get { return false; }
            }

            public void ApplySort(PropertyDescriptor property, ListSortDirection direction)
            {
                throw new Exception("The method or operation is not implemented.");
            }

            public int Find(PropertyDescriptor property, object key)
            {
                throw new Exception("The method or operation is not implemented.");
            }

            public bool IsSorted
            {
                get { return false; }
            }

            public event ListChangedEventHandler ListChanged;

            public void RemoveIndex(PropertyDescriptor property)
            {
                throw new Exception("The method or operation is not implemented.");
            }

            public void RemoveSort()
            {
                throw new Exception("The method or operation is not implemented.");
            }

            public ListSortDirection SortDirection
            {
                get { throw new Exception("The method or operation is not implemented."); }
            }

            public PropertyDescriptor SortProperty
            {
                get { throw new Exception("The method or operation is not implemented."); }
            }

            public bool SupportsChangeNotification
            {
                get { return true; }
            }

            public bool SupportsSearching
            {
                get { return false; }
            }

            public bool SupportsSorting
            {
                get { return false; }
            }

            #endregion
        }
        #endregion
    }
}